@@ -9,21 +9,21 @@ MULTINOMIAL.SAMPLE.TIMES <- 11
 MINOR.correction <- 1e-1
 
 ### BOOTSTRAP.factor the cut off ratio of success times / total bootstrap times
-BOOTSTRAP.factor <- 0.1
+BOOTSTRAP.factor <- 0.4
 
 
 ### checking the input histogram in an appropariat format
-checking.hist <- function(hist)
+checking.hist <- function(n)
 {
-  if (ncol(hist)!=2 || is.numeric(hist[,1])==FALSE || is.numeric(hist[,2])==FALSE) {
+  if (ncol(n)!=2 || is.numeric(n[,1])==FALSE || is.numeric(n[,2])==FALSE) {
     stop("Input must be a two-column matrix")
   }
   ## the first column is the frequencies of observed items
-  freq <- hist[, 1]
+  freq <- n[, 1]
 
   ## the second column is the number of observed distinct items for each
   ## frequency
-  number.items <- hist[, 2]
+  number.items <- n[, 2]
 
   ## check whether frequencies are at least one and the histogram is sorted
   for (i in 1:length(freq))
@@ -37,7 +37,7 @@ checking.hist <- function(hist)
           stop("The first column is not sorted in the ascending order")
     }
 
-  return(hist)
+  return(n)
 }
 
 
@@ -45,7 +45,7 @@ checking.hist <- function(hist)
 ### argument t
 preseqR.rfa.estimate <- function(CF, t)
 {
-  if (class(CF) != "RFA")
+  if (class(CF) != "CF")
     return()
   if (t < 1) 
     stop("the t should be at least one!")
@@ -68,12 +68,12 @@ preseqR.rfa.estimate <- function(CF, t)
 
 
 ### extrapolate given a histogram and a continued fraction
-preseqR.extrapolate.distinct <- function(hist, CF, start.size = NULL,
+preseqR.extrapolate.distinct <- function(n, CF, start.size = NULL,
      step.size = NULL, max.size = NULL)
 {
-  checking.hist(hist)
+  checking.hist(n)
   ## check CF is a continued fraction with CF attribute
-  if (class(CF) != "RFA")
+  if (class(CF) != "CF")
     return(NULL)
 
   ## parameters for calling the c-encode function c_extrapolate_distinct
@@ -83,7 +83,7 @@ preseqR.extrapolate.distinct <- function(hist, CF, start.size = NULL,
   di <- as.integer(0)
   de <- as.integer(CF$degree)
 
-  total.sample <- floor(hist[, 1] %*% hist[, 2])
+  total.sample <- floor(n[, 1] %*% n[, 2])
 
   ## set start.size, step.size, max.size if they are not defined by user
   if (is.null(start.size))
@@ -113,11 +113,12 @@ preseqR.extrapolate.distinct <- function(hist, CF, start.size = NULL,
             estimate = as.double(vector(mode = 'numeric', extrap.size)),
             estimate.l = as.integer(0));
 
-  initial_sum <- floor(sum(hist[, 2]))
+  initial_sum <- floor(sum(n[, 2]))
   extrapolation <- out$estimate[ 1:out$estimate.l ] + initial_sum
 
   ## sample size vector for extrapolation
-  sample.size <- total.sample * (start.size + step.size*((1:length(extrapolation)) - 1)) + total.sample
+  sample.size <- total.sample * (start.size + step.size*((1:length(extrapolation)) - 1)) + 
+                 total.sample
   ## estimation should be conservative
   sample.size <- ceiling(sample.size)
 
@@ -128,80 +129,43 @@ preseqR.extrapolate.distinct <- function(hist, CF, start.size = NULL,
 }
 
 
-### do without replacement of random sampling given a count vector of a
-### histogram; size is a user defined sample size
-nonreplace.sampling <- function(size, hist)
+## sampling without replacement
+## n frequencies counts
+nonreplace.sampling <- function(size, n)
 {
   ## make sure frequencies are integers
-  hist[, 2] <- floor(hist[, 2])
+  n[, 2] <- floor(n[, 2])
   ## the number of distinct items
-  distinct <- sum(hist[, 2])
+  distinct <- sum(n[, 2])
 
   ## identifier for each distinct item
   ind <- 1:distinct
 
   ## the size of each read in the library
-  n <- rep(hist[, 1], hist[, 2])
+  N <- rep(n[, 1], n[, 2])
 
   ## construct a sample space X 
   ## the whole library represents by its indexes. If a read presents t
   ## times in the library, its indexes presents t times in X
-  X <- rep(ind, n)
+  X <- rep(ind, N)
 
   return(sample(X, size, replace = FALSE))
 }
 
-### sub sampling without replacement based on a histogram
+
+## sampling without replacement
+## input frequencies counts; output subsample as a frequencies counts
 preseqR.nonreplace.sampling <- function(size, n)
 {
-  hist <- n
-
   ## check the input histogram file
-  checking.hist(hist)
+  checking.hist(n)
   ## sub sampling
-  T <- nonreplace.sampling(size, hist)
+  X <- nonreplace.sampling(size, n)
   ## record the freq of each sampled species
-  hist[, 2] <- floor(hist[, 2])
-	distinct <- sum(hist[, 2])
-	X <- vector(length=as.integer(distinct), mode="numeric")
-	for (i in T) {
-    X[i] <- X[i] + 1
-  }
-  ## construct the corresponding histogram
-	H <- vector(length=max(X), mode="numeric")
-  for (i in X) {
-		if (i > 0) {
-			H[i] <- H[i] + 1
-    }
-  }
-  ind <- which(H != 0)
-	hist <- matrix(c(ind, H[ind]), ncol = 2)
-  colnames(hist) <- c("j", "n_j")
-  return(hist)
-}	
-
-
-### the function samples n histograms given a count vector of the histogram
-### it is based on sampling with replacement (multinomial distribution)
-replace.sampling <- function(n, hist)
-{
-  ## calculate the distinct number of sample
-  distinct <- sum(hist[, 2])
-
-  ## returning sampling matrix
-  ## each column of the matrix represent the second column of a histogram
-  ## all histograms use nonzero.index as the first column
-  return( rmultinom(n, distinct, hist[, 2]) )
-}
-
-
-### given a sample vector, the function counts the number of distinct molecules
-count.distinct <- function(sample)
-{
-  max.value <- max(sample)
-  sample.table <- vector(mode = "numeric", length = max.value)
-  sample.table[sample] <- 1
-  return(sum(sample.table))
+  freq.counts <- hist(X, breaks=0:max(X), plot=FALSE)$count
+  ## frequencies counts; frequency 0 excluded
+  T <- hist(freq.counts, breaks=-1:max(freq.counts), plot=FALSE)$counts[-1]
+  matrix(c(which(T != 0), T[which(T != 0)]), byrow = FALSE, ncol=2)
 }
 
 
@@ -209,15 +173,13 @@ count.distinct <- function(sample)
 ### the initial experiment
 preseqR.interpolate.distinct <- function(ss, n)
 {
-  hist <- n
-
-  checking.hist(hist)
+  checking.hist(n)
 
   ## calculate total number of sample
-  total.sample <- hist[, 1] %*% hist[, 2]
+  total.sample <- n[, 1] %*% n[, 2]
   N <- floor(total.sample)
 
-  initial.distinct <- sum(as.numeric(hist[, 2]))
+  initial.distinct <- sum(as.numeric(n[, 2]))
   ## the total individuals captured
   step.size <- as.double(ss)
 
@@ -231,21 +193,21 @@ preseqR.interpolate.distinct <- function(ss, n)
 
   ## explicit calculating the expectation for sampling without replacement
   ## see K.L Heck 1975
-  ## N is the size of population; n is the size of the sample;
+  ## N is the size of population;   
   ## S is the number of unique species
-  ## n is the size of the sub sample
-  expect.distinct <- function(hist, N, n, S) {
-    denom <- lchoose(N, n)
-    numer <- lchoose(N - hist[, 1], n)
+  ## x is the size of the sub sample
+  expect.distinct <- function(n, N, x, S) {
+    denom <- lchoose(N, x)
+    numer <- lchoose(N - n[, 1], x)
     p <- exp(numer - denom)
-    return(S - p %*% hist[, 2])
+    return(S - p %*% n[, 2])
   }
 
   ## sample size vector
   x <- step.size * ( 1:l )
 
   ## calculate the number of distinct reads based on each sample size
-  yield.estimates <- sapply(x, function(x) expect.distinct(hist, N, x, initial.distinct))
+  yield.estimates <- sapply(x, function(x) expect.distinct(n, N, x, initial.distinct))
 
   ## put sample.size and yield.estimates together into a matrix
   result <- matrix(c(x, yield.estimates), ncol = 2, byrow = FALSE)
@@ -255,9 +217,9 @@ preseqR.interpolate.distinct <- function(ss, n)
 }
 
 ### check the goodness of the sample based on good Good & Toulmin's model
-goodtoulmin.2x.extrap <- function(hist)
+goodtoulmin.2x.extrap <- function(n)
 {
-  return((-1)^(hist[, 1] + 1) %*% hist[, 2])
+  return((-1)^(n[, 1] + 1) %*% n[, 2])
 }
 
 
@@ -267,9 +229,7 @@ goodtoulmin.2x.extrap <- function(hist)
 preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
                               max.extrapolation = NULL, asym.linear=FALSE)
 {
-  hist <- n
-
-  checking.hist(hist)
+  checking.hist(n)
   ## setting the diagonal value
   di = 0
   ## minimum required number of terms of power series in order to construct
@@ -277,7 +237,7 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
   MIN_REQUIRED_TERMS <- 4
 
   ## calculate total number of sample
-  total.sample <- hist[, 1] %*% hist[, 2]
+  total.sample <- n[, 1] %*% n[, 2]
   total.sample <- floor(total.sample)
 
   ## set step.size as the size of the initial experiment if it is undefined
@@ -301,7 +261,7 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
   } else {
       ## interpolation when sample size is no more than total sample size
       ## interpolate and set the size of sample for initial extrapolation
-      out <- preseqR.interpolate.distinct(step.size, hist)
+      out <- preseqR.interpolate.distinct(step.size, n)
       yield.estimates <- out[, 2]
 
       ## starting sample size for extrapolation
@@ -314,8 +274,8 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
   }
 
   ## transform a histogram into a vector of frequencies
-  hist.count <- vector(length=max(hist[, 1]), mode="numeric")
-  hist.count[hist[, 1]] <- hist[, 2]
+  hist.count <- vector(length=max(n[, 1]), mode="numeric")
+  hist.count[n[, 1]] <- n[, 2]
   ## only use non zeros items in histogram from begining up to the first zero
   counts.before.first.zero = 1
   while (as.integer(counts.before.first.zero) <= length(hist.count) &&
@@ -327,9 +287,9 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
   mt <- min(mt, counts.before.first.zero - 1)
   if (asym.linear == TRUE && (mt %% 2 == 0)) {
     mt = mt - 1
-	} else {
+  } else {
     mt = mt - (mt %% 2)
-	}
+  }
 
   ## pre-check to make sure the sample is good for prediction
   if (mt < MIN_REQUIRED_TERMS)
@@ -339,7 +299,7 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
     write(m, stderr())
     return(NULL)
   }
-  if(goodtoulmin.2x.extrap(hist) < 0.0)
+  if(goodtoulmin.2x.extrap(n) < 0.0)
   {
     m <- paste("Library expected to saturate in doubling of size",
                " unable to extrapolate", sep = ',')
@@ -378,7 +338,7 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
   length(out$offset.coeffs) <- as.integer(abs(out$diagonal.idx))
   CF <- list(out$ps.coeffs, out$cf.coeffs, out$degree)
   names(CF) <- c('ps.coeffs', 'cf.coeffs', 'degree')
-  class(CF) <- 'RFA'
+  class(CF) <- 'CF'
 
   ## if the sample size is larger than max.extrapolation
   ## stop extrapolation
@@ -396,7 +356,7 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
   start <- ( starting.size - total.sample ) / total.sample
   end <- (max.extrapolation + MINOR.correction - total.sample) / total.sample
   step <- step.size / total.sample
-  res <- preseqR.extrapolate.distinct(hist, CF, start, step, end)
+  res <- preseqR.extrapolate.distinct(n, CF, start, step, end)
 
   ## combine results from interpolation/extrapolation
   yield.estimates <- c(yield.estimates, res[, 2])
@@ -413,19 +373,17 @@ preseqR.rfa.curve <- function(n, mt = 100, ss = NULL,
 
 ### generate complexity curve through bootstrapping the histogram
 preseqR.rfa.species.accum.curve <- function(
-    n, bootstrap.times = 100, mt = 100, ss = NULL,
+    n, bootstrap.times = 20, mt = 100, ss = NULL,
     max.extrapolation = NULL, conf = 0.95, asym.linear=FALSE)
 {
-  hist <- n
-
-  checking.hist(hist)
-  hist[, 2] <- floor(hist[, 2])
+  checking.hist(n)
+  n[, 2] <- floor(n[, 2])
  
   ## setting the diagonal value
   di = 0
 
   ## calculate the total number of sample
-  total.sample <- hist[, 1] %*% hist[, 2]
+  total.sample <- n[, 1] %*% n[, 2]
 
   ## set the step.size to the size of the initial experiment if it is undefined
   if (is.null(ss)) {
@@ -446,7 +404,7 @@ preseqR.rfa.species.accum.curve <- function(
   }
 
   ## record second columns of resampled histograms
-  re.hist.second.col <- matrix(data = 0, nrow = length(hist[, 1]),
+  re.hist.second.col <- matrix(data = 0, nrow = length(n[, 1]),
                                ncol = MULTINOMIAL.SAMPLE.TIMES)
 
   ## the number of resampling times
@@ -461,15 +419,16 @@ preseqR.rfa.species.accum.curve <- function(
   {
     ## combine nonzero.index column and the second column to build a histogram
     ## table
-    hist.table <- matrix(c(hist[, 1], x), ncol = 2, byrow = FALSE)
+    hist.table <- matrix(c(n[, 1], x), ncol = 2, byrow = FALSE)
     preseqR.rfa.curve(hist.table, mt, step.size, max.extrapolation, asym.linear=asym.linear)
   }
 
-  while (bootstrap.times > 0) {
+  BOOTSTRAP.times = bootstrap.times
 
+  while (bootstrap.times > 0) {
     ## do sampling with replacement
     ## re.hist.second.col saves the second columns of each resampled histogram
-    re.hist.second.col <- replace.sampling(MULTINOMIAL.SAMPLE.TIMES, hist)
+    re.hist.second.col <- rmultinom(MULTINOMIAL.SAMPLE.TIMES, sum(n[, 2]), n[, 2])
 
     ## estimate for each histogram
     out <- apply(re.hist.second.col, 2, f)
@@ -479,7 +438,7 @@ preseqR.rfa.species.accum.curve <- function(
     ## extract yields estimation from each estimation result.
     yields <- sapply(out, function(x) x$estimates[, 2])
 
-    if ( !is.null( dim(yields) ) )
+    if ( length(yields) > 0 )
     {
       ## update sampling status
       success.times <- dim(yields)[2]
@@ -496,6 +455,10 @@ preseqR.rfa.species.accum.curve <- function(
   ## enough successful sampling
   if (bootstrap.times <= 0) {
 
+    if (BOOTSTRAP.times < 30) {
+      warning("The confidence interval is not reliable because of insufficient iterations of bootstrapping.
+  Set the variable bootstrap.times at least 30 in order to construct confidence intervals")}
+
     ## the number of sampled points for complexity curve
     n <- dim(yield.estimates)[1]
 
@@ -526,7 +489,7 @@ preseqR.rfa.species.accum.curve <- function(
   }
 }
 
-print.RFA <- function(x, digit = 4, ...)
+print.CF <- function(x, digit = 4, ...)
 {
   s <- "CONTINUED FRACTION APPROXIMATION :\n\n"
 
@@ -534,12 +497,12 @@ print.RFA <- function(x, digit = 4, ...)
   s <- paste(s, "DEGREE\t", toString(x$degree), "\n\n", sep = '')
 
   ## print the diagonal value
-##  s <- paste(s, "DIAGONAL VALUE\t", toString(x$diagonal.idx), "\n\n", sep = '')
+  ## s <- paste(s, "DIAGONAL VALUE\t", toString(x$diagonal.idx), "\n\n", sep = '')
 
   ## print the coefficients depending on the value of diagonal value
   s <- paste(s, "COEFFICIENTS:\n\n", sep = '')
 
-##  di <- abs(x$diagonal.idx)
+  ##  di <- abs(x$diagonal.idx)
   di <- 0
     
   ## the function to print a coefficient
