@@ -37,13 +37,11 @@ preseqR.simu.hist <- function(L=1e8, size, FUN) {
   }
   nonzero.index <- which(hist.count != 0)
   nonzero <- hist.count[nonzero.index]
-  hist <- matrix(c(nonzero.index, nonzero), ncol = 2)
-  colnames(hist) <- c("n", "n_j")
-  return(hist) 
+  matrix(c(nonzero.index, nonzero), ncol = 2)
 }
 
 ### simulating an interpolation curve
-preseqR.simu.interpolate <- function(L=1e7, ss, max.size, k, FUN) {
+preseqR.simu.interpolate <- function(L=1e7, ss, max.size, r, FUN) {
   ## too much memory if L is too large
   if (L > 1e8) {
     L <- 1e8
@@ -75,46 +73,14 @@ preseqR.simu.interpolate <- function(L=1e7, ss, max.size, k, FUN) {
       }
     }
     ## add the new point into the interpolation curve
-    point <- c(1:length(hist.count) %*% hist.count, sum(hist.count[k:length(hist.count)]))
+    point <- c(1:length(hist.count) %*% hist.count, 
+               sum(hist.count[r:length(hist.count)]))
     points <- rbind(points, point)
     ## add the new generated histogram
     nonzero.index <- which(hist.count != 0)
     nonzero <- hist.count[nonzero.index]
-    hist <- matrix(c(nonzero.index, nonzero), ncol = 2)
-    colnames(hist) <- c("n", "n_j")
-    hists <- c(hists, list(hist))
-  }
-  res <- list(histograms = hists, interpolation.curve = unname(points))
-  return(res)
-}
-
-### Construct the histogram given the size of the initial sample,
-### the underlining Gamma-Poisson distribution and the total number of distinct
-### species. Truncated the frequency when it is smaller than one and all frequencies
-### are the integeral parts of calculated expectations.
-simu.hist <- function(L, alpha, beta, n) {
-  ## coefficients of the constructed power series
-  PS <- c()
-  ## the ratio between the size of the sample to the expected size of an initial sample
-  t <- n / (L * alpha * beta)
-  ## set a upperbound for the number of items in the power series
-  mt <- 100
-
-  i = 1
-  while (i < mt) {
-    co.eff = L * dnbinom(x=i, size=alpha, prob=1/(1 + t*beta))
-    if (floor(co.eff) < 1) {
-      break
-    } else {
-      PS <- c(PS, floor(co.eff))
-    }
-    i <- i + 1
-  }
-  l <- length(PS)
-  if (l > 0) {
-    hist <- matrix(c(1:l, PS), byrow=FALSE, ncol=2)
-    return(hist)
-  } else {
-   return(NULL)
+    n <- matrix(c(nonzero.index, nonzero), ncol = 2)
+    hists <- c(hists, list(n))
   }
+  list(histograms = hists, interpolation.curve = unname(points))
 }
